import pytest

from moviebox_api.v1.models import SuggestedItemsModel
from moviebox_api.v2.core import (
    SearchSuggestion,
    Session,
)
from tests.v2 import MOVIE_KEYWORD


@pytest.mark.asyncio
async def test_search_suggestion():
    suggestion = SearchSuggestion(Session())
    suggestion_details = await suggestion.get_content_model(MOVIE_KEYWORD)
    assert isinstance(suggestion_details, SuggestedItemsModel)



import pytest

from moviebox_api.v2.constants import SubjectType
from moviebox_api.v2.core import Search, SingleItemDetails, TVSeriesDetails
from moviebox_api.v2.models import SearchResultsModel, SpecificItemDetailsModel
from moviebox_api.v2.requests import Session
from tests.v2 import TV_SERIES_KEYWORD


@pytest.mark.parametrize(
    argnames=["path"],
    argvalues=(["goat-u1jZhR4CnV4"],),
)
def test_single_item_using_detail_path(path):
    session = Session()
    details = SingleItemDetails(
        session=session,
    )
    assert type(details.get_content_sync(path)) is dict
    content = details.get_content_model_sync(path)

    assert isinstance(content, SpecificItemDetailsModel)


@pytest.mark.parametrize(
    argnames=["path"],
    argvalues=(["warrior-Au4u1Uu2Nf3"],),
)
def test_tv_series_using_detail_path(path):
    session = Session()
    details = TVSeriesDetails(
        session=session,
    )

    assert type(details.get_content_sync(path)) is dict

    assert isinstance(
        details.get_content_model_sync(path), SpecificItemDetailsModel
    )


@pytest.mark.parametrize(
    argnames=["keyword", "subject_type"],
    argvalues=(
        ["Titanic", SubjectType.MOVIES],
        ["Walker", SubjectType.MUSIC],
        # ["King", SubjectType.ANIME],
        ["War", SubjectType.EDUCATION],
    ),
)
def test_single_item_using_search_results_item(
    keyword: str, subject_type: SubjectType
):
    session = Session()

    search = Search(session, query=keyword, subject_type=subject_type)

    search_results = search.get_content_model_sync()

    assert isinstance(search_results, SearchResultsModel)

    details = SingleItemDetails(
        session=session,
    )
    assert type(details.get_content_sync(search_results.first_item)) is dict

    assert isinstance(
        details.get_content_model_sync(search_results.first_item),
        SpecificItemDetailsModel,
    )


def test_tv_series_using_search_results_item():
    session = Session()
    search = Search(
        session,
        query=TV_SERIES_KEYWORD,
        subject_type=SubjectType.TV_SERIES,
    )
    search_results = search.get_content_model_sync()

    assert isinstance(search_results, SearchResultsModel)

    details = TVSeriesDetails(
        session=session,
    )
    assert (
        type(
            details.get_content_sync(
                search_results.first_item,
            )
        )
        is dict
    )

    assert isinstance(
        details.get_content_model_sync(
            search_results.first_item,
        ),
        SpecificItemDetailsModel,
    )



from moviebox_api.v1 import MovieDetails, Session, TVSeriesDetails
from tests.v1 import TEST_MOVIE_PAGE_URL, TEST_TV_SERIES_PAGE_URL, project_dir

content_names = ["content_path"]

MOVIE_CONTENT_PATH = project_dir / "assets/data/movie.page"
TV_SERIES_CONTENT_PATH = project_dir / "assets/data/tv_series.page"

content_paths = (
    [MOVIE_CONTENT_PATH],
    [TV_SERIES_CONTENT_PATH],
)


def read_content(path):
    with open(path, encoding="utf-8") as fh:
        return fh.read()


def write_content(path, content):
    with open(path, 'w') as fh:
        fh.write(content)


def update_movie_contents():
    session = Session()
    md = MovieDetails(TEST_MOVIE_PAGE_URL, session)

    content = md.get_html_content_sync()

    write_content(MOVIE_CONTENT_PATH, content)


def update_tv_series_contents():
    session = Session()
    md = TVSeriesDetails(TEST_TV_SERIES_PAGE_URL, session)

    content = md.get_html_content_sync()

    write_content(TV_SERIES_CONTENT_PATH, content)


update_movie_contents()
update_tv_series_contents()


"""
This module contains base classes for the entire package
"""

import asyncio
import os
from abc import ABC, abstractmethod
from pathlib import Path

import httpx
from throttlebuster import DownloadedFile

from moviebox_api.utils import get_event_loop


class BaseMovieboxException(Exception):
    """Base class for all exceptions of this package"""


class BaseContentProvider(ABC):
    """Provides easy retrieval of resource from moviebox"""

    @abstractmethod
    async def get_content(self, *args, **kwargs) -> dict | list[dict]:
        """Response as received from server"""
        raise NotImplementedError("Function needs to be implemented in subclass.")

    @abstractmethod
    async def get_content_model(self, *args, **kwargs) -> object | list[object]:
        """Modelled version of the content"""
        raise NotImplementedError("Function needs to be implemented in subclass.")


class ContentProviderHelper:
    """Provides common methods to content provider classes"""

    def get_content_sync(self, *args, **kwargs) -> dict | list[dict]:
        """Get content `synchronously`"""
        return get_event_loop().run_until_complete(
            self.get_content(*args, **kwargs)
        )

    def get_content_model_sync(self, *args, **kwargs) -> object | list[object]:
        """Get content model `synchronously`"""
        return get_event_loop().run_until_complete(
            self.get_content_model(*args, **kwargs)
        )


class BaseContentProviderAndHelper(BaseContentProvider, ContentProviderHelper):
    """A class that inherits both `BaseContentProvider(ABC)` and
    `ContentProviderHelper`"""


class BaseFileDownloader(ABC):
    """Base class for media and caption files downloader"""

    @abstractmethod
    async def run(self, *args, **kwargs) -> DownloadedFile | httpx.Response:
        """Downloads a movie or caption file"""
        raise NotImplementedError("Function needs to be implemented in subclass.")


class FileDownloaderHelper:
    """Provide common method to file downloaders"""

    def run_sync(self, *args, **kwargs) -> DownloadedFile | httpx.Response:
        """Sychronously performs the actual download"""
        return asyncio.get_event_loop().run_until_complete(
            self.run(*args, **kwargs)
        )


class BaseFileDownloaderAndHelper(FileDownloaderHelper, BaseFileDownloader):
    """Inherits both `FileDownloaderHelper` and `BaseFileDownloader`"""

    @classmethod
    def create_final_dir(
        cls,
        working_dir: Path,
        search_results_item: object,  # "SearchResultsItem",
        season: int,
        episode: int,
        test: bool,
        group: bool,
    ):
        if group and season and episode:
            # series it is
            working_dir = Path(working_dir)
            assert working_dir.exists(), (
                f"The chosen working directory does not exist - {working_dir}"
            )

            final_dir = working_dir.joinpath(
                f"{search_results_item.title} "
                f"({search_results_item.releaseDate.year})",
                f"S{season}",
            )

            if not test:
                os.makedirs(final_dir, exist_ok=True)

            return final_dir

        return working_dir



import pytest

from moviebox_api.v2.core import (
    Search,
    SubjectType,
)
from moviebox_api.v2.models import SearchResultsModel
from moviebox_api.v2.requests import Session
from tests.v2 import init_search


@pytest.mark.parametrize(
    argnames=["subject_type"],
    argvalues=(
        [SubjectType.ALL],
        [SubjectType.MOVIES],
        [SubjectType.TV_SERIES],
        [SubjectType.MUSIC],
        [SubjectType.EDUCATION],
        [SubjectType.ANIME],
    ),
)
def test_get_content_and_model(subject_type: SubjectType):
    search: Search = init_search(Session(), subject_type=subject_type)

    contents = search.get_content_sync()
    assert type(contents) is dict
    modelled_contents = search.get_content_model_sync()

    assert isinstance(modelled_contents, SearchResultsModel)
    
    if subject_type == SubjectType.ALL:
        return
    
    for item in modelled_contents.items:
        assert item.subjectType == subject_type


def test_next_page_navigation():
    search = init_search(Session())
    contents = search.get_content_model_sync()
    assert isinstance(contents, SearchResultsModel)

    next_search = search.next_page(contents)
    assert isinstance(next_search, Search)
    next_contents = next_search.get_content_model_sync()

    assert isinstance(next_contents, SearchResultsModel)
    assert contents.pager.page + 1 == next_contents.pager.page


def test_previous_page_navigation():
    search: Search = init_search(Session(), page=3)
    contents = search.get_content_model_sync()
    assert isinstance(contents, SearchResultsModel)

    previous_search = search.previous_page(contents)
    assert isinstance(previous_search, Search)
    previous_contents = previous_search.get_content_model_sync()

    assert isinstance(previous_contents, SearchResultsModel)
    assert contents.pager.page - 1 == previous_contents.pager.page



import asyncio


def build_command_group(click_group, commands_map: dict[callable, str]):

    for cmd_func, name in commands_map.items():
        click_group.add_command(cmd_func, name)

    return click_group


def get_event_loop():
    try:
        event_loop = asyncio.get_event_loop()
    except RuntimeError:
        event_loop = asyncio.new_event_loop()
        asyncio.set_event_loop(event_loop)
    return event_loop